from collections import Mapping
from operator import itemgetter as _itemgetter
import heapq as _heapq
from itertools import repeat as _repeat, chain as _chain, starmap as _starmap
from typing import Union as _tUnion, Iterable as _tIterable

from sortedcontainers import SortedDict as _SortedDict


def _count_elements(mapping, iterable):
    """Tally elements from the iterable."""
    mapping_get = mapping.get
    for elem in iterable:
        mapping[elem] = mapping_get(elem, 0) + 1


try:  # Load C helper function if available
    from _collections import _count_elements
except ImportError:
    pass


class SortedCounter(_SortedDict):
    """Dict subclass for counting hashable items.  Sometimes called a bag
    or multiset.  Elements are stored as dictionary keys and their counts
    are stored as dictionary values.
    >>> c = SortedCounter('abcdeabcdabcaba')  # count elements from a string
    >>> c.most_common(3)                # three most common elements
    [('a', 5), ('b', 4), ('c', 3)]
    >>> sorted(c)                       # list all unique elements
    ['a', 'b', 'c', 'd', 'e']
    >>> ''.join(sorted(c.elements()))   # list elements with repetitions
    'aaaaabbbbcccdde'
    >>> sum(c.values())                 # total of all counts
    15
    >>> c['a']                          # count of letter 'a'
    5
    >>> for elem in 'shazam':           # update counts from an iterable
    ...     c[elem] += 1                # by adding 1 to each element's count
    >>> c['a']                          # now there are seven 'a'
    7
    >>> del c['b']                      # remove all 'b'
    >>> c['b']                          # now there are zero 'b'
    0
    >>> d = SortedCounter('simsalabim')       # make another counter
    >>> c.update(d)                     # add in the second counter
    >>> c['a']                          # now there are nine 'a'
    9
    >>> c.clear()                       # empty the counter
    >>> c
    Counter()
    Note:  If a count is set to zero or reduced to zero, it will remain
    in the counter until the entry is deleted or the counter is cleared:
    >>> c = SortedCounter('aaabbc')
    >>> c['b'] -= 2                     # reduce the count of 'b' by two
    >>> c.most_common()                 # 'b' is still in, but its count is zero
    [('a', 3), ('c', 1), ('b', 0)]
    """

    # References:
    #   http://en.wikipedia.org/wiki/Multiset
    #   http://www.gnu.org/software/smalltalk/manual-base/html_node/Bag.html
    #   http://www.demo2s.com/Tutorial/Cpp/0380__set-multiset/Catalog0380__set-multiset.htm
    #   http://code.activestate.com/recipes/259174/
    #   Knuth, TAOCP Vol. II section 4.6.3

    def __init__(*args, **kwds):
        """Create a new, empty SortedCounter object.  And if given, count elements
        from an input iterable.  Or, initialize the count from another mapping
        of elements to their counts.
        >>> c = SortedCounter()                           # a new, empty counter
        >>> c = SortedCounter('gallahad')                 # a new counter from an iterable
        >>> c = SortedCounter({'a': 4, 'b': 2})           # a new counter from a mapping
        >>> c = SortedCounter(a=4, b=2)                   # a new counter from keyword args
        """
        if not args:
            raise TypeError("descriptor '__init__' of 'SortedCounter' object "
                            "needs an argument")
        self, *args = args
        if len(args) > 1:
            raise TypeError('expected at most 1 arguments, got %d' % len(args))
        super(SortedCounter, self).__init__()
        self.update(*args, **kwds)

    def __missing__(self, key):
        """The count of elements not in the SortedCounter is zero."""
        # Needed so that self[missing_item] does not raise KeyError
        return 0

    def most_common(self, n=None):
        """List the n most common elements and their counts from the most
        common to the least.  If n is None, then list all element counts.
        >>> SortedCounter('abcdeabcdabcaba').most_common(3)
        [('a', 5), ('b', 4), ('c', 3)]
        """
        # Emulate Bag.sortedByCount from Smalltalk
        if n is None:
            return sorted(self.items(), key=_itemgetter(1), reverse=True)
        return _heapq.nlargest(n, self.items(), key=_itemgetter(1))

    def elements(self):
        """Iterator over elements repeating each as many times as its count.
        >>> c = SortedCounter('ABCABC')
        >>> sorted(c.elements())
        ['A', 'A', 'B', 'B', 'C', 'C']
        # Knuth's example for prime factors of 1836:  2**2 * 3**3 * 17**1
        >>> prime_factors = SortedCounter({2: 2, 3: 3, 17: 1})
        >>> product = 1
        >>> for factor in prime_factors.elements():     # loop over factors
        ...     product *= factor                       # and multiply them
        >>> product
        1836
        Note, if an element's count has been set to zero or is a negative
        number, elements() will ignore it.
        """
        # Emulate Bag.do from Smalltalk and Multiset.begin from C++.
        return _chain.from_iterable(_starmap(_repeat, self.items()))

    # Override dict methods where necessary

    @classmethod
    def fromkeys(cls, iterable, v=None):
        # There is no equivalent method for counters because setting v=1
        # means that no element can have a count greater than one.
        raise NotImplementedError(
            'SortedCounter.fromkeys() is undefined.  Use SortedCounter(iterable) instead.')

    def update(*args, **kwds):
        """Like dict.update() but add counts instead of replacing them.
        Source can be an iterable, a dictionary, or another Counter instance.
        >>> c = SortedCounter('which')
        >>> c.update('witch')           # add elements from another iterable
        >>> d = SortedCounter('watch')
        >>> c.update(d)                 # add elements from another counter
        >>> c['h']                      # four 'h' in which, witch, and watch
        4
        """
        # The regular dict.update() operation makes no sense here because the
        # replace behavior results in the some of original untouched counts
        # being mixed-in with all of the other counts for a mismash that
        # doesn't have a straight-forward interpretation in most counting
        # contexts.  Instead, we implement straight-addition.  Both the inputs
        # and outputs are allowed to contain zero and negative counts.

        if not args:
            raise TypeError("descriptor 'update' of 'SortedCounter' object "
                            "needs an argument")
        self, *args = args
        if len(args) > 1:
            raise TypeError('expected at most 1 arguments, got %d' % len(args))
        iterable = args[0] if args else None
        if iterable is not None:
            if isinstance(iterable, Mapping):
                if self:
                    self_get = self.get
                    for elem, count in iterable.items():
                        self[elem] = count + self_get(elem, 0)
                else:
                    super(SortedCounter, self).update(iterable)  # fast path when counter is empty
            else:
                _count_elements(self, iterable)
        if kwds:
            self.update(kwds)

    def subtract(*args, **kwds):
        """Like dict.update() but subtracts counts instead of replacing them.
        Counts can be reduced below zero.  Both the inputs and outputs are
        allowed to contain zero and negative counts.
        Source can be an iterable, a dictionary, or another SortedCounter or Counter instance.
        >>> c = SortedCounter('which')
        >>> c.subtract('witch')             # subtract elements from another iterable
        >>> c.subtract(SortedCounter('watch'))    # subtract elements from another counter
        >>> c['h']                          # 2 in which, minus 1 in witch, minus 1 in watch
        0
        >>> c['w']                          # 1 in which, minus 1 in witch, minus 1 in watch
        -1
        """
        if not args:
            raise TypeError("descriptor 'subtract' of 'SortedCounter' object "
                            "needs an argument")
        self, *args = args
        if len(args) > 1:
            raise TypeError('expected at most 1 arguments, got %d' % len(args))
        iterable: _tUnion[Mapping, _tIterable] = args[0] if args else None
        if iterable is not None:
            self_get = self.get
            if isinstance(iterable, Mapping):
                for elem, count in iterable.items():
                    self[elem] = self_get(elem, 0) - count
            else:
                for elem in iterable:
                    self[elem] = self_get(elem, 0) - 1
        if kwds:
            self.subtract(kwds)

    def copy(self):
        """Return a shallow copy."""
        return self.__class__(self)

    def __reduce__(self):
        return self.__class__, (dict(self),)

    def __delitem__(self, elem):
        """Like dict.__delitem__() but does not raise KeyError for missing values."""
        if elem in self:
            super().__delitem__(elem)

    def __repr__(self):
        if not self:
            return '%s()' % self.__class__.__name__
        try:
            items = ', '.join(map('%r: %r'.__mod__, self.most_common()))
            return '%s({%s})' % (self.__class__.__name__, items)
        except TypeError:
            # handle case where values are not orderable
            return '{0}({1!r})'.format(self.__class__.__name__, dict(self))

    # Multiset-style mathematical operations discussed in:
    #       Knuth TAOCP Volume II section 4.6.3 exercise 19
    #       and at http://en.wikipedia.org/wiki/Multiset
    #
    # Outputs guaranteed to only include positive counts.
    #
    # To strip negative and zero counts, add-in an empty counter:
    #       c += SortedCounter()

    def __add__(self, other):
        """Add counts from two counters.
        >>> SortedCounter('abbb') + SortedCounter('bcc')
        SortedCounter({'b': 4, 'c': 2, 'a': 1})
        """
        if not isinstance(other, SortedCounter):
            return NotImplemented
        result = SortedCounter()
        for elem, count in self.items():
            newcount = count + other[elem]
            if newcount > 0:
                result[elem] = newcount
        for elem, count in other.items():
            if elem not in self and count > 0:
                result[elem] = count
        return result

    def __sub__(self, other):
        """ Subtract count, but keep only results with positive counts.
        >>> SortedCounter('abbbc') - SortedCounter('bccd')
        SortedCounter({'b': 2, 'a': 1})
        """
        if not isinstance(other, SortedCounter):
            return NotImplemented
        result = SortedCounter()
        for elem, count in self.items():
            newcount = count - other[elem]
            if newcount > 0:
                result[elem] = newcount
        for elem, count in other.items():
            if elem not in self and count < 0:
                result[elem] = 0 - count
        return result

    def __or__(self, other):
        """Union is the maximum of value in either of the input counters.
        >>> SortedCounter('abbb') | SortedCounter('bcc')
        SortedCounter({'b': 3, 'c': 2, 'a': 1})
        """
        if not isinstance(other, SortedCounter):
            return NotImplemented
        result = SortedCounter()
        for elem, count in self.items():
            other_count = other[elem]
            newcount = other_count if count < other_count else count
            if newcount > 0:
                result[elem] = newcount
        for elem, count in other.items():
            if elem not in self and count > 0:
                result[elem] = count
        return result

    def __and__(self, other):
        """ Intersection is the minimum of corresponding counts.
        >>> SortedCounter('abbb') & SortedCounter('bcc')
        SortedCounter({'b': 1})
        """
        if not isinstance(other, SortedCounter):
            return NotImplemented
        result = SortedCounter()
        for elem, count in self.items():
            other_count = other[elem]
            newcount = count if count < other_count else other_count
            if newcount > 0:
                result[elem] = newcount
        return result

    def __pos__(self):
        """Adds an empty counter, effectively stripping negative and zero counts"""
        result = SortedCounter()
        for elem, count in self.items():
            if count > 0:
                result[elem] = count
        return result

    def __neg__(self):
        """Subtracts from an empty counter.  Strips positive and zero counts,
        and flips the sign on negative counts.
        """
        result = SortedCounter()
        for elem, count in self.items():
            if count < 0:
                result[elem] = 0 - count
        return result

    def _keep_positive(self):
        """Internal method to strip elements with a negative or zero count"""
        nonpositive = [elem for elem, count in self.items() if not count > 0]
        for elem in nonpositive:
            del self[elem]
        return self

    def __iadd__(self, other):
        """Inplace add from another counter, keeping only positive counts.
        >>> c = SortedCounter('abbb')
        >>> c += SortedCounter('bcc')
        >>> c
        SortedCounter({'b': 4, 'c': 2, 'a': 1})
        """
        for elem, count in other.items():
            self[elem] += count
        return self._keep_positive()

    def __isub__(self, other):
        """Inplace subtract counter, but keep only results with positive counts.
        >>> c = SortedCounter('abbbc')
        >>> c -= SortedCounter('bccd')
        >>> c
        SortedCounter({'b': 2, 'a': 1})
        """
        for elem, count in other.items():
            self[elem] -= count
        return self._keep_positive()

    def __ior__(self, other):
        """Inplace union is the maximum of value from either counter.
        >>> c = SortedCounter('abbb')
        >>> c |= SortedCounter('bcc')
        >>> c
        SortedCounter({'b': 3, 'c': 2, 'a': 1})
        """
        for elem, other_count in other.items():
            count = self[elem]
            if other_count > count:
                self[elem] = other_count
        return self._keep_positive()

    def __iand__(self, other):
        """Inplace intersection is the minimum of corresponding counts.
        >>> c = SortedCounter('abbb')
        >>> c &= SortedCounter('bcc')
        >>> c
        SortedCounter({'b': 1})
        """
        for elem, count in self.items():
            other_count = other[elem]
            if other_count < count:
                self[elem] = other_count
        return self._keep_positive()
